<?php
require_once '/var/packages/Spreadsheet/target/phpoffice/word/Autoloader.php';

\PhpOffice\PhpWord\Autoloader::register();
\PhpOffice\PhpWord\Settings::loadConfig();

// ===== for debugging =====================
$ob_file = fopen('/var/log/synoffice.log','a');

function ob_file_callback($buffer)
{
	if ($buffer !== '') {
		global $ob_file;
		fwrite($ob_file, "doc import: " . $buffer);
	}
}

ob_start('ob_file_callback');
//======================

function print_log($msg) {
	$line = "doc import@" . Date('Y-m-d H:i:s') . ": $msg\n";
	file_put_contents('/var/log/synoffice.log', $line, FILE_APPEND);
}

function print_log_array($ary) {
	foreach ($ary as $k => $v) {
		print_log("{$k}:{$v}");
	}
}

function extract_syno_json($path) {
	$zip = new ZipArchive();
	if ($zip->open($path) !== TRUE) {
		return NULL;
	}

	$json_string = $zip->getFromName('synoDoc.xml');
	$zip->close();

	if ($json_string === FALSE) {
		return NULL;
	}

	$xmlReader = new \PhpOffice\PhpWord\Shared\XMLReader();
	$xmlReader->getDomFromZip($path, '/docProps/core.xml');
	$created = $xmlReader->getElement('//dcterms:created');
	$modified = $xmlReader->getElement('//dcterms:modified');
	if (is_null($created) || is_null($modified) || $created->nodeValue !== $modified->nodeValue) {
		return NULL;
	}

	return convert_syno_json(json_decode($json_string));
}

function arrange_json_attachments($docxPath, $attachment) {
	global $synoTempPath;
	$extractPath = "{$synoTempPath}/docx";
	mkdir($extractPath, 0777, true);
	$zip = new ZipArchive();
	$zip->open($docxPath);
	$zip->extractTo($extractPath);
	$zip->close();

	if (is_object($attachment) || is_array($attachment)) {
		$configMap = array();
		foreach ($attachment as $fileId => $config) {
			$configMap[$fileId] = $config;
		}
		// media
		if ( file_exists("{$extractPath}/word/media") ) {
			$media = scandir("{$extractPath}/word/media");
			foreach ($media as $fileName) {
				$pos = strrpos($fileName, '.');
				$fileId = (false !== $pos) ? substr($fileName, 0, $pos) : $fileName;
				$config = $configMap[$fileId];
				if ( isset($config) ) {
					rename("{$extractPath}/word/media/{$fileName}",
						"{$synoTempPath}/{$fileId}:{$config->basename}.{$config->ext}");
				}
			}
		}
		// embeddings
		if ( file_exists("{$extractPath}/word/embeddings") ) {
			$embeddings = scandir("{$extractPath}/word/embeddings");
			foreach ($embeddings as $fileName) {
				$pos = strrpos($fileName, '.');
				$fileId = (false !== $pos) ? substr($fileName, 0, $pos) : $fileName;
				$config = $configMap[$fileId];
				if ( isset($config) ) {
					$oleObject = new \PhpOffice\PhpWord\Element\Object("{$extractPath}/word/embeddings/{$fileName}");
					$fout = fopen("{$extractPath}/word/embeddings/{$config->basename}.{$config->ext}", 'wb', false);
					if (false !== $fout) {
						fwrite($fout, $oleObject->getObjectStringData());
						fclose($fout);
					}
					rename("{$extractPath}/word/embeddings/{$config->basename}.{$config->ext}",
						"{$synoTempPath}/{$fileId}:{$config->basename}.{$config->ext}");
				}
			}
		}
	}
}

function convert_comments() {
	global $comment;
	global $paraIdMap;
	global $paraIdRelation;
	$comments = array();
	if (is_array($comment)) {
		foreach ($comment as $c) {
			$replies = array();

			if (isset($c['paraId']) && is_array($paraIdRelation[$c['paraId']]) ) {
				foreach ( $paraIdRelation[$c['paraId']] as $r ) {
					$rid = $paraIdMap[$r];
					$replies[] = array(
						'id' => $rid,
						'author' => $comment[$rid]['author'],
						'timestamp' => null,
						'text' => $comment[$rid]['text'],
						'status' => 0, // open
						'replies' => null,
						'lastModified' => null,
					);
				}
			}

			$comments[] = array(
				'type' => $c['type'],
				'id' => $c['id'],
				'author' => $c['author'],
				'timestamp' => null,
				'text' => $c['text'],
				'status' => 0, // open
				'replies' => $replies,
				'lastModified' => null,
			);
		}
	}
	return $comments;
}

function check_pm_doc($json) {
	return ($json && is_string($json->type) && is_array($json->content))? true : false;
}

function check_comment($json) {
	return is_array($json)? true : false;
}

function convert_syno_json($json) {
	if (!check_pm_doc($json->content->data)) {
		$json->content->data = $json->content->json;
		unset($json->content->json);
	}
	if (!check_comment($json->comment->data)) {
		$json->comment->data = $json->comment->json;
		unset($json->comment->json);
	}
	return $json;
}

function main($argv) {
	global $synoTempPath;

	$src = $argv[1];
	$dst = $argv[2];
	$synoTempPath = $argv[3] .'/_data/';
	$fileType = $argv[4];
	mkdir($synoTempPath, 0777, true);

	$name = '';
	if ('docx' === $fileType) {
		$name = 'Word2007';
	} else if ('odt' === $fileType) {
		$name = 'ODText';
	} else if ('rtf' === $fileType) {
		$name = 'RTF';
	}

	$synoJson = extract_syno_json($src);
	if (!is_null($synoJson)) {
		file_put_contents($dst, json_encode($synoJson));
		arrange_json_attachments($src, $synoJson->attachment);
		//////file_put_contents('/var/log/import.json', json_encode($synoJson));
	} else {
		$phpword = \PhpOffice\PhpWord\IOFactory::load($src, $name);
		$phpword->save($dst, 'HTML_ProseMirror');
		//////$phpword->save('/var/log/import.html', 'HTML_ProseMirror');

		$comments = convert_comments();
		file_put_contents($dst . '.comment', json_encode($comments));
		//////file_put_contents('/var/log/import.html.comment', json_encode($comments));
	}
}

main($argv);
?>
