<?php
require_once '/var/packages/Spreadsheet/target/phpoffice/common/Autoloader.php';
require_once '/var/packages/Spreadsheet/target/phpoffice/presentation/Autoloader.php';

\PhpOffice\Common\Autoloader::register();
\PhpOffice\PhpPresentation\Autoloader::register();

$ob_file = fopen('/var/log/synoffice.log','a');

function ob_file_callback($buffer)
{
	if ($buffer !== '') {
		global $ob_file;
		fwrite($ob_file,"slides import: " . $buffer);
	}
}

ob_start('ob_file_callback');

function check_pm_doc($json) {
	return ($json && is_string($json->type) && is_array($json->content))? true : false;
}

function check_comment($json) {
	return is_array($json)? true : false;
}

function convert_syno_json($json) {
	if ($json->content) {
		if (check_pm_doc($json->content)) {
			$syno_content = new \stdClass();
			$syno_content->data = $json->content;
			$json->content = $syno_content;
		} else if (!check_pm_doc($json->content->data)) {
			$json->content->data = $json->content->json;
			unset($json->content->json);
		}
	}
	if ($json->comment) {
		if (check_comment($json->comment)) {
			$syno_comment = new \stdClass();
			$syno_comment->data = $json->comment;
			$json->comment = $syno_comment;
		} else if (!check_comment($json->comment->data)) {
			$json->comment->data = $json->comment->json;
			unset($json->comment->json);
		}
	}
	return $json;
}

function main($argv) {
	global $synoTempPath;
	global $comment;

	$src = $argv[1];
	$dst = $argv[2];
	$synoTempPath = $argv[3] . '/_data/';
	mkdir($synoTempPath, 0777, true);

	$zip = new ZipArchive();
	$zip->open($src);
	$jsonString = $zip->getFromName('synoPresentation.json');
	$zip->close();
	$json = convert_syno_json(json_decode($jsonString));

	$content = new \stdClass();
	$content->data = $json->content? $json->content->data : new \stdClass();
	$content->version = 0;

	$comment = new \stdClass();
	$comment->data = $json->comment? $json->comment->data : array();
	$comment->format = 1204;
	$comment->version = 0;

	$wrapper = new \stdClass();
	$wrapper->content = $content;
	$wrapper->comment = $comment;

	file_put_contents($dst, json_encode($wrapper));
	if (!is_null($json->created)) {
		$created = intval($json->created);
	}

	$phpPresentation = \PhpOffice\PhpPresentation\IOFactory::load($src);
	$writer = new \PhpOffice\PhpPresentation\Writer\ProseMirror($phpPresentation);

	if ($created !== $phpPresentation->getDocumentProperties()->getModified()) {
		$writer->save($dst);
	} else {
		$extractPath = "$synoTempPath/pptx";
		mkdir($extractPath, 0777, true);
		$zip->open($src);
		$zip->extractTo($extractPath);
		$zip->close();

		$attachment = $json->attachment;
		if (is_object($attachment) || is_array($attachment)) {
			$attachmentConfigMap = array();
			foreach ($attachment as $fileId => $config) {
				$attachmentConfigMap[$fileId] = $config;
			}

			$images = scandir("$extractPath/ppt/media");
			foreach ($images as $fileName) {
				$pos = strrpos($fileName, '.');
				$fileId = (false !== $pos) ? substr($fileName, 0, $pos) : $fileName;
				$config = $attachmentConfigMap[$fileId];
				if (!isset($config)) {
					continue;
				}
				rename("$extractPath/ppt/media/$fileName", "$synoTempPath/$fileId:" . $config->basename);
			}
		}
	}

	if (file_exists('/tmp/gen_slide_testcase')) {
		mkdir('/tmp/slide_testcase/import', 0700, true);
		$i = 1;
		do {
			if (file_exists("/tmp/slide_testcase/import/case$i")) {
				$i++;
				continue;
			}
			break;
		} while (true);
		mkdir("/tmp/slide_testcase/import/case$i", 0700);
		copy($dst, "/tmp/slide_testcase/import/case$i/test.json");
		copy($src, "/tmp/slide_testcase/import/case$i/test.pptx");
	}

	chown($dst, 'office');
	chgrp($dst, 'office');
}

main($argv);
?>
