<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element;

/**
 * TextBreak element HTML writer
 *
 * @since 0.10.0
 */
class TextBreak extends AbstractElement
{
    /**
     * Write text break
     *
     * @return string
     */
    public function write()
    {
        // writing hr if border exists
        $pStyle = $this->element->getParagraphStyle();
        if (method_exists($pStyle, 'hasBorder') && $pStyle->hasBorder()) {
            return '<div><hr></div>';
        }

        // setup paragraph style
        $style = "";
        if (method_exists($pStyle, 'getSize')) {
            $size = $pStyle->getSize();
            if (!is_null($size)) {
                $style .= "font-size:{$size}pt;";
            }
        }
        if (!empty($style)) {
            $style = " style=" . $style;
        }

        if ($this->withoutP) {
            $content = '<br />' . PHP_EOL;
        } else {
            if (method_exists($pStyle, 'getRevision')) {
                $revision = $pStyle->getRevision();
                $style .= " data-syno-comment=\"{$revision['id']}\"";
                $style .= ' data-syno-suggestion="' . array('del' => 'deletion', 'ins' => 'insertion')[$revision['op']] . '"';
            }
            $content = "<p{$style}>&nbsp;</p>" . PHP_EOL;
        }

        return $content;
    }
}
